/*
  Copyright (c) 2013, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include "mmss.h"

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>


int main() {
	mmss_context_t ctx = { .now = 0 };

	mmss_network_t net = { .mtu = 1500 };
	gmrf_context_t node1 = { .mmss = &ctx }, node2 = { .mmss = &ctx };
	gmrf_iface_t iface1 = {}, iface2 = {};

	iface1.net = &net;
	iface1.node = &node1;
	iface1.address = (gmrf_addr_t){{1}};

	iface2.net = &net;
	iface2.node = &node2;
	iface2.address = (gmrf_addr_t){{2}};

	node1.interfaces = &iface1;
	node2.interfaces = &iface2;

	iface2.network_next = &iface1;
	net.interfaces = &iface2;

	node2.next = &node1;
	gmrf_context_t *nodes = &node2;

	while (true) {
		int timeout = mmss_queue_timeout(&ctx, &ctx.packet_queue);

		if (timeout < 0) {
			fprintf(stderr, "Nothing queued, deadlock occured.\n");
			break;
		}

		if (timeout > 0) {
			assert(!mmss_queue_get(&ctx, &ctx.packet_queue));

			ctx.now += timeout;
			timeout = mmss_queue_timeout(&ctx, &ctx.packet_queue);
		}

		assert(timeout == 0);

		while (timeout == 0) {
			mmss_packet_t *packet = mmss_queue_get(&ctx, &ctx.packet_queue);
			assert(packet);

			packet->dest->node->handle_packet(packet->dest->node, packet->dest,
							  &packet->source->address, packet->data,
							  packet->len);

			free(packet);

			timeout = mmss_queue_timeout(&ctx, &ctx.packet_queue);
		}
	}

	return 0;
}
